/*=========================================================================

   Program: ParaView
   Module:    qcMainWindow.cxx

   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
   All rights reserved.

   ParaView is a free software; you can redistribute it and/or modify it
   under the terms of the ParaView license version 1.2.

   See License_v1.2.txt for the full ParaView license.
   A copy of this license can be obtained by contacting
   Kitware Inc.
   28 Corporate Drive
   Clifton Park, NY 12065
   USA

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

========================================================================*/
#include "qcMainWindow.h"
#include "ui_qcMainWindow.h"

#include "pqActiveObjects.h"
#include "pqApplicationCore.h"
#include "pqLoadDataReaction.h"
#include "pqObjectBuilder.h"
#include "pqOutputPort.h"
#include "pqParaViewBehaviors.h"
#include "pqPipelineSource.h"
#include "pqRenderView.h"
#include "pqServer.h"
#include "vtkSMProxyManager.h"
#include "vtkSMReaderFactory.h"

//-----------------------------------------------------------------------------
qcMainWindow::qcMainWindow(QWidget *_parent, Qt::WindowFlags _flags):
  Superclass(_parent, _flags)
{
  // Setup the UI.
  Ui::MainWindow ui;
  ui.setupUi(this);

  // We want this application to behave like ParaView for the most part e.g. was
  // window state when application quits, connect the built-in server by
  // default etc. Hence we simply instantiate this class.
  new pqParaViewBehaviors(this, this);

  // We want the actionLoad to result in the showing up the ParaView's OpenData
  // dialog letting the user pick from one of the supported file formats.
  pqLoadDataReaction* dataLoader = new pqLoadDataReaction(ui.actionLoad);
  QObject::connect(dataLoader, SIGNAL(loadedData(pqPipelineSource*)),
    this, SLOT(onDataLoaded(pqPipelineSource*)));

  // Initialize all readers available to ParaView. Now our application can load
  // all types of datasets supported by ParaView.
  vtkSMProxyManager::GetProxyManager()->GetReaderFactory()->RegisterPrototypes("sources");

  // Create the two views.
  this->View1 = this->createRenderView(ui.renderFrame1);
  this->View2 = this->createRenderView(ui.renderFrame2);
}

//-----------------------------------------------------------------------------
qcMainWindow::~qcMainWindow()
{
}

//-----------------------------------------------------------------------------
pqRenderView* qcMainWindow::createRenderView(QWidget* widget)
{
  QHBoxLayout *hbox = new QHBoxLayout(widget);
  hbox->setMargin(0);

  // Create a new render view.
  pqObjectBuilder* builder = pqApplicationCore::instance()->getObjectBuilder();
  pqRenderView *view = qobject_cast<pqRenderView*>(
    builder->createView(pqRenderView::renderViewType(),
      pqActiveObjects::instance().activeServer()));

  // Place the widget from for the render view in the frame provided.
  hbox->addWidget(view->getWidget());
  return view;
}

//-----------------------------------------------------------------------------
void qcMainWindow::onDataLoaded(pqPipelineSource* source)
{
  pqObjectBuilder* builder = pqApplicationCore::instance()->getObjectBuilder();
  // Show the data in the two views.
  builder->createDataRepresentation(source->getOutputPort(0), this->View1);
  builder->createDataRepresentation(source->getOutputPort(0), this->View2);

  // Reset the camera to ensure that the data is visible.
  this->View1->resetDisplay();
  this->View2->resetDisplay();

  // Trigger renders.
  this->View1->render();
  this->View2->render();
}
